"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const pjson = require('../package.json');
const util_1 = require("util");
const deps_1 = require("./deps");
const deprecatedCLI = util_1.deprecate(() => {
    return require('cli-ux').cli;
}, 'this.out and this.cli is deprecated. Please import the "cli-ux" module directly instead.');
class Command {
    constructor(config) {
        this.config = config;
        this.flags = {};
        this.args = {};
        if (deps_1.default.HTTP) {
            this.http = deps_1.default.HTTP.defaults({
                headers: {
                    'user-agent': `${this.config.name}/${this.config.version} (${this.config.platform}-${this.config.arch}) node-${process.version}`,
                },
            });
        }
    }
    static buildHelp(config) {
        let help = new deps_1.default.Help(config);
        return help.command(this);
    }
    static buildHelpLine(config) {
        let help = new deps_1.default.Help(config);
        return help.commandLine(this);
    }
    get ctor() {
        return this.constructor;
    }
    async init(argv) {
        const parse = await deps_1.default.CLIFlags.parse({
            argv,
            args: this.ctor.args || [],
            flags: this.ctor.flags || {},
            strict: !this.ctor.variableArgs,
        });
        this.flags = parse.flags;
        this.argv = parse.argv;
        this.args = parse.args;
    }
    async done() {
        try {
            if (deps_1.default.cli)
                await deps_1.default.cli.done();
        }
        catch (err) {
            if (deps_1.default.cli)
                deps_1.default.cli.warn(err);
            else
                // tslint:disable-next-line
                console.error(err);
        }
    }
}
Command.aliases = [];
Command.variableArgs = false;
Command.args = [];
// tslint:disable-next-line
Command._version = pjson.version;
/**
 * instantiate and run the command setting {mock: true} in the config (shorthand method)
 */
Command.mock = async function (argv = [], config = {}) {
    if (typeof argv === 'string') {
        // old-style call
        util_1.deprecate(() => {
            argv = Array.from(arguments);
            config = {};
        }, "`Command.mock('--foo', 'bar')` is deprecated. Please use `Command.mock(['--foo', 'bar'])` instead.")();
    }
    if (deps_1.default.cli)
        deps_1.default.cli.config.mock = true;
    const cmd = await this.run(argv, new deps_1.default.Config(config));
    return {
        cmd,
        get out() {
            return deprecatedCLI();
        },
        stderr: deps_1.default.cli ? deps_1.default.cli.stderr.output : 'cli-ux not found',
        stdout: deps_1.default.cli ? deps_1.default.cli.stdout.output : 'cli-ux not found',
    };
};
/**
 * instantiate and run the command
 */
Command.run = async function (argv = [], config) {
    const cmd = new this(config);
    try {
        await cmd.init(argv);
        await cmd.run();
        await cmd.done();
    }
    catch (err) {
        if (!deps_1.default.cli)
            throw err;
        deps_1.default.cli.error(err);
    }
    return cmd;
};
exports.Command = Command;
